local huds = { "wolfhud", "sydneyhud", "mui", "cshud" }
local function UpdateOneItem(self, menu, item_to_update, status)
    for _, item in ipairs(menu.items) do
        if item.id == item_to_update then
            self:AnimateItemEnabled(item, status)
            break
        end
    end
end

local function UpdateText(self, text)
    local menu = self:GetMenu("btp_menu")
    for _, item in pairs(menu.items) do
        if item.id == "btp_compatibility_text" then
            item.panel:child("title"):set_text(text)
            break
        end
    end
end

local function CheckHUDCompatibility(self, menu, h)
    local condition = h ~= 1 -- 1 = WolfHUD
    local not_mui = h ~= 3 -- 3 = MUI
    local not_csgo = h ~= 4 -- 4 = CS:GO HUD
    local not_mui_and_not_csgo = not_mui and not_csgo
    UpdateOneItem(self, menu, "btp_panel_style_choice", not_mui_and_not_csgo)
    UpdateOneItem(self, menu, "btp_panel_style_position_choice", not_mui_and_not_csgo)
    UpdateOneItem(self, menu, "btp_steam_avatar_compatibility_choice", not_mui_and_not_csgo)
    UpdateOneItem(self, menu, "btp_radial_rip_color_choice", not_mui_and_not_csgo)
    UpdateOneItem(self, menu, "btp_stoic_health_choice", not_mui_and_not_csgo)
    UpdateOneItem(self, menu, "btp_show_stamina_counter_choice", condition)
    UpdateOneItem(self, menu, "btp_stamina_counter_color_choice", condition and BTP:GetOption("show_stamina_counter"))
end

local function UpdateHUDItems(self, menu, value)
    if type(menu) == "string" then
        menu = self:GetMenu(menu)
    end
    for _, item in ipairs(menu.items) do
        for _, v in ipairs(huds) do
            if item.id == v then
                self:AnimateItemEnabled(item, false)
            end
        end
    end
    local h = 0
    if value == 1 then -- Autodetect
        if WolfHUD then
            h = 1
        elseif SydneyHUD then
            h = 2
        elseif MUI then
            h = 3
        elseif CSHUD then
            h = 4
        end
    elseif value == 3 then -- WolfHUD
        h = 1
    elseif value == 4 then -- SydneyHUD
        h = 2
    elseif value == 5 then -- MUI
        h = 3
    elseif value == 6 then -- CS HUD
        h = 4
    end
    CheckHUDCompatibility(self, menu, h)
    if h == 0 then
        return
    end
    for _, item in pairs(menu.items) do
        if item.id == huds[h] then
            self:AnimateItemEnabled(item, true)
        end
    end
end

local function Update(self, menu, items_to_update, status)
    local item_id
    for _, item in pairs(menu.items) do
        item_id = item.id
        for _, v in pairs(items_to_update) do
            if item_id == v then
                self:AnimateItemEnabled(item, status)
            end
        end
    end
end

local function UpdateItemsInMenu(self, hud)
    local panel = BTP:GetOption("panel_style")
    local menu = self:GetMenu("btp_menu")
    local not_default = panel ~= 1
    local replace = BTP:GetOption("panel_style_position") == 1
    local not_mui = hud ~= 5
    local not_csgo = hud ~= 6
    UpdateOneItem(self, menu, "btp_steam_avatar_compatibility_choice", panel == 2 and not_mui and not_csgo)
    UpdateOneItem(self, menu, "btp_panel_style_position_choice", not_default and not_mui and not_csgo)
    UpdateOneItem(self, menu, "btp_armor_color_choice", not_default and replace and not_mui and not_csgo)
    UpdateOneItem(self, menu, "btp_radial_rip_color_choice", not_default and replace and not_mui and not_csgo)
    UpdateOneItem(self, menu, "btp_stoic_health_choice", not_default and replace and not_mui and not_csgo)
    UpdateOneItem(self, menu, "btp_heister_avatar_style_choice", not_mui and not_csgo and panel == 3)
end

local function UpdateCompatibilityText(self, hud, detection)
    local str = detection and "btp_compatibility_detected" or "btp_compatibility_forced"
    str = managers.localization:text(str) .. managers.localization:text("btp_compatibility_" .. hud)
    UpdateText(self, str)
end

local function GetCorrectCompatibilityNumber(self)
    local hud = BTP:GetOption("hud_compatibility")
    local detection = false
    if hud == 1 then
        if WolfHUD then
            hud = 3
        elseif SydneyHUD then
            hud = 4
        elseif MUI then
            hud = 5
        elseif CSHUD then
            hud = 6
        else
            hud = 2
        end
        detection = true
    end
    UpdateCompatibilityText(self, hud, detection)
    return hud
end

function BTPMenu:UpdateItems()
    UpdateItemsInMenu(self, GetCorrectCompatibilityNumber(self))
    --[[if hud == 1 then -- Autodetect
        if WolfHUD and WolfHUD:getSetting({"CustomHUD", "ENABLED"}, true) then
        end
    else
        if hud == 2 then -- Vanilla
        else -- WolfHUD

        end
    end]]
end

-- Main Menu
function BTPMenu:BTPMenuCreated()
    UpdateItemsInMenu(self, GetCorrectCompatibilityNumber(self))
    UpdateHUDItems(self, self:GetMenu("btp_menu"), BTP:GetOption("hud_compatibility"))
end

function BTPMenu:SetBTPCompatibility(value)
    UpdateHUDItems(self, self:GetMenu("btp_menu"), value)
end

function BTPMenu:SetOption(value, option)
    BTP.settings[option] = value
end

function BTPMenu:SetColor(color, option)
    local c = BTP.settings[option]
    c.r = color.r
    c.g = color.g
    c.b = color.b
end

function BTPMenu:callback_btp_mod_language(value)
    BTP.settings.mod_language = value
end

-- SydneyHUD
function BTPMenu:callback_btp_sydneyhud_stamina_bar_position(value)
    BTP.settings.hud.sydneyhud.stamina_bar_position = value
end