local BTP = BTP
local stamina_counter_visibility = BTP:GetOption("show_stamina_counter")
local ping_counter_visibility = BTP:GetOption("show_ping_counter")
local avatar_position = BTP:GetOption("panel_style_position")
local avatar_replace = avatar_position == 1
local function SetFontSize(o, f)
    o:set_font_size(f)

	local _, _, w_rect, h_rect = o:text_rect()
	o:set_size(w_rect, h_rect)
end

local function AlignRight(img, text)
    text:set_center(img:center())
    text:set_left(img:right())
end

function MUITeammate:InitBTP()
    local btp_panel = managers.hud._mui_hud.panel:panel({
        name = "btp_" .. self._id
    })
    if self._main_player then
        self:CreateStaminaCounter()
    else
        self:CreatePingCounter()
    end
    self:ResizeBTPPanels()
end

function MUITeammate:CreateStaminaCounter()
    self._stamina_counter = self._panel:panel({
        name = "stamina_counter",
        layer = 1,
        visible = stamina_counter_visibility
    })
    local stamina_icon = self._stamina_counter:bitmap({
        name = "stamina_icon",
        texture = tweak_data.btp.textures.stamina_counter.texture,
        texture_rect = tweak_data.btp.textures.stamina_counter.texture_rect,
        layer = 10,
        w = 16,
        h = 16,
        x = 0,
        visible = true
    })
    local stamina_text = self._stamina_counter:text({
        name = "stamina_text",
        layer = 10,
        text = "100%",
        w = 16,
        h = 16,
        x = 16,
        y = 2.5,
        visible = true,
        font_size = 14,
        font = self._font,
        color = Color.green
    })
    managers.hud:make_fine_text(stamina_text)
end

function MUITeammate:CreatePingCounter()
    self._ping_counter = self._panel:panel({
        name = "ping_counter",
        layer = 1,
        visible = false
    })
    local ping_icon = self._ping_counter:bitmap({
        name = "ping_icon",
        texture = tweak_data.btp.textures.ping_counter.texture,
        texture_rect = tweak_data.btp.textures.ping_counter.texture_rect,
        layer = 10,
        w = 16,
        h = 16,
        x = 0,
        visible = true
    })
    local ping_text = self._ping_counter:text({
        name = "ping_text",
        layer = 10,
        text = "0000ms",
        w = 16,
        h = 16,
        x = 16,
        y = 2.5,
        visible = true,
        font_size = 14,
        font = self._font,
        color = Color.white
    })
    managers.hud:make_fine_text(ping_text)
    self:ResetPingText()
end

function MUITeammate:SetMaxStamina(value)
    self._max_stamina = value
end

local function sign(v)
    return (v >= 0 and 1) or -1
end

local function round(v, bracket)
    bracket = bracket or 1
    return math.floor(v / bracket + sign(v) * 0.5) * bracket
end

if BTP:GetOption("stamina_counter_color") then
    function MUITeammate:SetStaminaValue(value)
        local v = value / self._max_stamina
        self._stamina_counter:child("stamina_text"):set_text((round(v, 0.01) * 100) .. "%")
        self._stamina_counter:child("stamina_text"):set_color(Color(1, 1 - (1 * v), 1 * v, 0))
    end
else
    function MUITeammate:SetStaminaValue(value)
        self._stamina_counter:child("stamina_text"):set_text((round(value / self._max_stamina, 0.01) * 100) .. "%")
    end
end

local _f_resize = MUITeammate.resize
function MUITeammate:resize()
    _f_resize(self)
    self:ResizeBTPPanels()
end

function MUITeammate:ResizeBTPPanels()
    local panel = self._panel
    --self._panel:set_h(self._panel:h() + 28)
    --self._panel:set_x(self._panel:x() + 28)

    local stamina_counter = self._stamina_counter
    local ping_counter = self._ping_counter
    --[[local brk = break_time_panel
    local spawns = spawns_panel
    local captain = captain_panel]]
    local main = self._main_player
	local size = main and self._muiSizeL or self._muiSizeS
    local s33 = size / 3
    local s332 = s33 * 2
    local wMul = panel:w()
    local width = size * wMul

    if main then
        Figure(stamina_counter):shape(width, s332)
        local stamina_i = stamina_counter:child("stamina_icon")
        local stamina_t = stamina_counter:child("stamina_text")

        SetFontSize(stamina_t, s33)
        AlignRight(stamina_i, stamina_t)
    else
        Figure(ping_counter):shape(width, s332)
        local ping_i = ping_counter:child("ping_icon")
        local ping_t = ping_counter:child("ping_text")

        SetFontSize(ping_t, s33)
        AlignRight(ping_i, ping_t)
    end

    --[[local time_i = time:child("icon") -- Time Left Icon
    local time_t = time:child("text") -- "Time Left:" Text
    local break_i = brk:child("icon") -- Break Time Left Icon
    local break_t = brk:child("text") -- "Break Time Left: " Text
    local captain_i = captain:child("icon") -- Captain Icon
    local captain_t = captain:child("amount") -- Captain Text
    local spawns_i = spawns:child("icon") -- Spawns Left Icon
    local spawns_t = spawns:child("text") -- "Spawns Left:" Text
    local loot_panel = panel:child("loot_panel")

    SetFontSize(time_left, s33)
    SetFontSize(time_t, s33)
    AlignRight(time_i, time_t)
    AlignRight(time_t, time_left)

    SetFontSize(break_left, s33)
    SetFontSize(break_t, s33)
    AlignRight(break_i, break_t)
    AlignRight(break_t, break_left)

    SetFontSize(spawns_left, s33)
    SetFontSize(spawns_t, s33)
    AlignRight(spawns_i, spawns_t)
    AlignRight(spawns_t, spawns_left)

    SetFontSize(captain_buff, s33)
    SetFontSize(captain_t, s33)
    AlignRight(captain_i, captain_t)
    AlignRight(captain_i, captain_buff)

    time:set_y(loot_panel:bottom())]]

    local offset = main and 48 or 32
    if stamina_counter_visibility and main then
        stamina_counter:set_x(panel:w() - offset)
    else
        ping_counter:set_x(panel:w() - offset)
    end
end

local _f_remove_panel = MUITeammate.remove_panel
function MUITeammate:remove_panel()
    self:ResetPingText()
    _f_remove_panel(self)
end

function MUITeammate:SetPingVisibility(visibility)
    if not self._main_player then
        self._ping_counter:set_visible(visibility and ping_counter_visibility)
    end
end

if BTP:GetOption("ping_counter_color") then
    function MUITeammate:SetPingText(ping, desync)
        if not self._main_player then
            self._ping_counter:child("ping_text"):set_text(ping .. "ms")
            self._ping_counter:child("ping_text"):set_color(BTP:GetPingColor(ping, desync))
        end
    end
else
    function MUITeammate:SetPingText(ping, desync)
        if not self._main_player then
            self._ping_counter:child("ping_text"):set_text(ping .. "ms")
        end
    end
end

function MUITeammate:ResetPingText()
    self:SetPingText(0)
end

function MUITeammate:ReloadSteamAvatar()
end

function MUITeammate:ActivateArmorBreakInvulnerability(t)
    self._radial_custom:stop()
    self._radial_custom:animate(function(o)
        o:set_color(Color(1, 1, 1, 1))
        o:set_visible(true)
        over(t, function(p)
            o:set_color(Color(1, 1 - p, 1, 1))
        end)
        o:set_visible(false)
    end)
end