PlayerInfoComponent.AvatarStatus = class(PlayerInfoComponent.Base)
function PlayerInfoComponent.AvatarStatus:init(panel, owner, width, height, scale, visibility, local_player, settings)
    PlayerInfoComponent.AvatarStatus.super.init(self, panel, owner, "avatar_status", width, height)

    self._is_local_player = not visibility

    self._main_player = local_player

    self._settings = settings

    self._size = height

    self._scale = scale

    if not WolfHUD:getSetting({"CustomHUD", "PLAYER", "STATUS"}, true) then
        self._panel:hide()
    end

    self._avatar = self._panel:bitmap({
        name = "avatar",
        texture = "guis/textures/pd2_mod_btp/avatar_none",
        render_template = "VertexColorTexturedRadial",
        h = self._size - (8 * scale),
        w = self._size - (8 * scale),
        x = 2 * scale,
        y = 2 * scale,
        layer = 10,
        visible = true
    })

    self._avatar_armor = self._panel:bitmap({
        name = "avatar_armor",
        texture = "guis/textures/pd2_mod_btp/hud_armor_" .. BTP:GetOption("armor_color"),
        layer = 12,
        w = self._size - (4 * scale),
        h = self._size - (4 * scale),
        render_template = "VertexColorTexturedRadial",
        color = Color(1, 0, 1, 1),
        visible = true
    })

    self._avatar_ability_meter = self._panel:bitmap({
        name = "ability_meter",
        texture = "guis/textures/pd2_mod_btp/hud_ability_radial",
        layer = 13,
        w = self._size - (4 * scale),
        h = self._size - (4 * scale),
        render_template = "VertexColorTexturedRadial",
        visible = false,
        color = Color(1, 0, 0, 0)
    })

    self._avatar_ability_icon = self._panel:bitmap({
        name = "ability_icon",
        layer = 13,
        w = self._avatar:w() * 0.5,
        h = self._avatar:h() * 0.5,
        visible = false
    })
    self._avatar_ability_icon:set_center(self._avatar:center())

    self._avatar_swan_song = self._panel:bitmap({
        name = "swan_song",
        texture = "guis/textures/pd2_mod_btp/hud_swan_song",
        layer = 14,
        w = self._size - (4 * scale),
        h = self._size - (4 * scale),
        render_template = "VertexColorTexturedRadial",
        visible = false,
        color = Color(1, 0, 0, 0)
    })

    self._avatar_swan_song_icon = self._panel:bitmap({
        name = "swan_song_icon",
        texture = "guis/textures/pd2/hud_swansong_icon",
        layer = 15,
        w = self._avatar:w() * 0.9,
        h = self._avatar:h() * 0.9,
        visible = false
    })
    self._avatar_swan_song_icon:set_center(self._avatar:center())

    self._stored_health_max = 0
    self._radial_rip = self._panel:bitmap({
        name = "avatar",
        texture = "guis/textures/pd2_mod_btp/hud_armor_" .. BTP:GetOption("radial_rip_color"),
        render_template = "VertexColorTexturedRadial",
        h = self._size - (8 * scale),
        w = self._size - (8 * scale),
        x = 2 * scale,
        y = 2 * scale,
        layer = 11,
        visible = true
    })

    self._components = { self._avatar, self._avatar_armor, self._avatar_ability_meter, self._avatar_ability_icon, self._avatar_swan_song, self._avatar_swan_song_icon }

    self.id = "AvatarStatus"

    -- Health
    self._owner:register_listener(self.id, { "health" }, callback(self, self, "set_health"), false)

    -- Armor
    self._owner:register_listener(self.id, { "armor" }, callback(self, self, "set_armor"), false)

    -- Ability Radial
    self._owner:register_listener(self.id, { "ability_icon" }, callback(self, self, "set_ability_icon"), false)
    self._owner:register_listener(self.id, { "ability_radial" }, callback(self, self, "set_ability_radial"), false)
    self._owner:register_listener(self.id, { "activate_ability" }, callback(self, self, "activate_ability_radial"), false)

    -- Swan Song Radial
    self._owner:register_listener(self.id, { "custom_radial" }, callback(self, self, "set_custom_progress"), false)

    -- Ex-President's Radial
    self._owner:register_listener(self.id, { "stored_health" }, callback(self, self, "set_stored_health"), false)
    self._owner:register_listener(self.id, { "stored_health_max" }, callback(self, self, "set_stored_health_max"), false)

    -- Invalidate Steam Avatar (dropin)
    self._owner:register_listener(self.id, { "ReloadAvatar" }, callback(self, self, "ReloadAvatar"), false)

    BTP:SetAvatar(self, callback(self, self, "SetAvatar"))
end

function PlayerInfoComponent.AvatarStatus:destroy()
    self._owner:unregister_listener(self.id, {
        "health", "armor", "ability_icon", "ability_radial", "activate_ability", "custom_radial", "stored_health", "stored_health_max", "ReloadAvatar"
    })
    PlayerInfoComponent.AvatarStatus.super.destroy(self)
end

function PlayerInfoComponent.AvatarStatus:set_health(current, total)
    local ratio = current / total
    self._avatar:stop()
    self._avatar:set_color(Color(1, ratio, 1, 1))
end

function PlayerInfoComponent.AvatarStatus:set_stored_health(amount)
    local red = math.min(amount, 1)
    self._radial_rip:set_color(Color(1, red / self._stored_health_max, 1, 1))
end

function PlayerInfoComponent.AvatarStatus:set_stored_health_max(amount)
    self._stored_health_max = amount
end

function PlayerInfoComponent.AvatarStatus:set_armor(current, total)
    local ratio = current / total
    self._avatar_armor:set_color(Color(1, ratio, 1, 1))
end

function PlayerInfoComponent.AvatarStatus:SetAvatar(texture)
    texture = texture or "guis/textures/pd2_mod_btp/avatar_none"
    self._avatar:set_image(texture)
end

function PlayerInfoComponent.AvatarStatus:rescale(factor)
    if PlayerInfoComponent.AvatarStatus.super.rescale(self, factor) then
        self._size = self._size * factor
        self._avatar:set_w(self._size - (8 * factor))
        self._avatar:set_h(self._size - (8 * factor))
        self._radial_rip:set_w(self._avatar:w())
        self._radial_rip:set_h(self._avatar:h())
        self._avatar_armor:set_w(self._size - (4 * factor))
        self._avatar_armor:set_h(self._size - (4 * factor))
        self._avatar_ability_meter:set_w(self._size - (4 * factor))
        self._avatar_ability_meter:set_h(self._size - (4 * factor))
        self._avatar_ability_icon:set_w(self._avatar:w() * 0.5)
        self._avatar_ability_icon:set_h(self._avatar:h() * 0.5)
        self._avatar_ability_icon:set_center(self._avatar:center())
        self._avatar_swan_song_icon:set_w(self._avatar:w() * 0.9)
        self._avatar_swan_song_icon:set_h(self._avatar:h() * 0.9)
        self._avatar_swan_song_icon:set_center(self._avatar:center())
        self._scale = self._scale * factor
    end
end

function PlayerInfoComponent.AvatarStatus:update_settings()
    if WolfHUD:getSetting({"CustomHUD", "PLAYER", "STATUS"}, true) then
        self._panel:show()
        self._owner:arrange()
    else
        self._panel:hide()
    end
end

function PlayerInfoComponent.AvatarStatus:set_ability_icon(icon)
    local texture, texture_rect = tweak_data.hud_icons:get_icon_data(icon, {
		0,
		0,
		32,
		32
    })

    self._avatar_ability_icon:set_image(texture, unpack(texture_rect))
end

function PlayerInfoComponent.AvatarStatus:set_ability_radial(current, total)
    local progress = current / total
    self._avatar_ability_meter:set_color(Color(1, progress, 1, 1))
    self._avatar_ability_icon:set_visible(progress > 0)
end

function PlayerInfoComponent.AvatarStatus:activate_ability_radial(time_left, time_total)
    local meter = self._avatar_ability_meter
    local icon = self._avatar_ability_icon

    local function anim()
        local progress_start = time_left / time_total

        icon:set_visible(true)

        over(time_left, function(p)
            local progress = progress_start * math.lerp(1, 0, p)

            meter:set_color(Color(1, progress, 1, 1))
        end)

        icon:set_visible(false)
    end

    meter:stop()
    meter:animate(anim)
end

function PlayerInfoComponent.AvatarStatus:set_custom_progress(current, total)
    local ratio = current / total
    self._avatar_swan_song:set_color(Color(1, ratio, 1, 1))
    self._avatar_swan_song:set_visible(ratio > 0)
    self._avatar_swan_song_icon:set_visible(ratio > 0)
end

function PlayerInfoComponent.AvatarStatus:set_is_local_player(state)
    self._main_player = state
    if state then
        self._panel:show()
    else
        self._panel:hide()
    end
end

function PlayerInfoComponent.AvatarStatus:ReloadAvatar()
    self:SetAvatar()
    self._steam_texture_obtained = nil
    BTP:SetAvatar(self, callback(self, self, "SetAvatar"))
end

function HUDTeammateCustom:InitBTP()
    local size = 50 * self._scale
    self._avatar = PlayerInfoComponent.AvatarStatus:new(self._panel, self, size, size, self._scale, self:peer_id() or self:is_ai(), self._is_player, self._settings)
    self._all_components[#self._all_components + 1] = self._avatar
    if self._is_player then
        self._player_status._panel:set_alpha(0)
    end
end

local _f_arrange = HUDTeammateCustom.arrange
function HUDTeammateCustom:arrange()
    _f_arrange(self)
    if not self._component_layout then return end

    self._avatar:set_y(self._player_status:y())
    self._avatar:set_x(self._player_status:x())
end

function HUDTeammateCustom:ReloadSteamAvatar()
    self:call_listeners("ReloadAvatar")
end

function HUDTeammateCustom:MainPlayerOverride()
end

function HUDTeammateCustom:SetMaxStamina(value)
end

function HUDTeammateCustom:SetStaminaValue(value)
end

function HUDTeammateCustom:SetPingVisibility(visibility)
end

function HUDTeammateCustom:SetPingText(ping)
end

function HUDTeammateCustom:ResetPingText()
end

function HUDTeammateCustom:ActivateArmorBreakInvulnerability(t)
end

function HUDTeammateCustom:SetArmorBreakInvulnerabilityVisibility(visibility, t)
end