if not RequiredScript then
    return
end

if RequiredScript == "lib/managers/hudmanager" then
    dofile(BTP.LuaPath .. "hudmanagerpd2.lua")
end

if RequiredScript == "core/lib/utils/coreapp" then
    if BTP then
        BTP = BTP or {}
        return
    end

    BTP = {
        _cache = {},

        Enum =
        {
            LogType =
            {
                Normal = 1,
                Warning = 2
            }
        },

        ModPath = ModPath,
        AssetsPath = ModPath .. "assets/",
        LocPath = ModPath .. "loc/",
        LuaPath = ModPath .. "lua/",
        PanelPath = ModPath .. "lua/panel/",
        MenuPath = ModPath .. "menu/",
        HostPath = ModPath .. "lua/host/",
        ClientPath = ModPath .. "lua/client/",
        CompatibilityPath = ModPath .. "lua/compatibility/",
        MenuCompatibilityPath = ModPath .. "menu/compatibility/",
        SettingsSaveFilePath = BLTModManager.Constants:SavesDirectory() .. "btp.json",
        ModVersion = ModInstance and tonumber(ModInstance:GetVersion()) or "N/A",
        Update = false,
        Language = "english",
        SaveDataVer = 1
    }

    function BTP:PreHook(object, func, pre_call)
        Hooks:PreHook(object, func, "BTP_Pre_" .. func, pre_call)
    end

    function BTP:Hook(object, func, post_call)
        Hooks:PostHook(object, func, "BTP_" .. func, post_call)
    end

    function BTP:Unhook(mod, id)
        Hooks:RemovePostHook((mod and (mod .. "_") or "BTP_") .. id)
    end

    function BTP:ApplyHUDCompatibility(hud)
        dofile(self.HUDCompatibilityPath .. hud .. ".lua")
    end

    function BTP:DelayCall(name, t, func)
        DelayedCalls:Add(name, t, func)
    end

    function BTP:RemoveDelayedCall(name)
        DelayedCalls:Remove(name)
    end

    function BTP:ApplyCompatibilityFile(hud)
        dofile(self.CompatibilityPath .. hud .. ".lua")
    end

    function BTP:Load()
        self:LoadDefaultValues()
        local file = io.open(self.SettingsSaveFilePath, "r")
        if file then
            local table = json.decode(file:read('*all')) or {}
            file:close()
            if table.SaveDataVer and table.SaveDataVer == self.SaveDataVer then
                self:LoadValues(self.settings, table)
            else
                self.SaveDataNotCompatible = true
                self:Save()
            end
        end
    end

    function BTP:LoadValues(btp_table, file_table)
        for k, v in pairs(file_table) do -- Load subtables in table and calls the same method to load subtables or values in that subtable
            if type(file_table[k]) == "table" and btp_table[k] then
                self:LoadValues(btp_table[k], v)
            end
        end
        for k, v in pairs(file_table) do
            if type(file_table[k]) ~= "table" then
                if btp_table and btp_table[k] ~= nil then -- Load values to the table
                    btp_table[k] = v
                end
            end
        end
    end

    function BTP:Save()
        self.settings.SaveDataVer = self.SaveDataVer
        self.settings.ModVersion = self.ModVersion
        local file = io.open(self.SettingsSaveFilePath, "w+")
        if file then
            file:write(json.encode(self.settings) or "{}")
            file:close()
        end
    end

    function BTP:LoadDefaultValues()
        local file = io.open(self.MenuPath .. "default_values.json", "r")
        if file then
            self.settings = json.decode(file:read('*all') or { mod_language = 1 })
            file:close()
        else
            self:Log("No default values were found! Game may crash unexpectedly!", self.Enum.LogType.Warning)
            self.settings = { mod_language = 1 }
        end
    end

    function BTP:CallEvent(event_name, ...)
        if not event_name then
            log("[BTP] Event Name cannot be a nil value")
            return
        end
        for _, v in pairs(self.Events) do
            if v.event_name == event_name then
                v.func(...)
            end
        end
    end

    function BTP:GetOption(option)
        if option then
            return self.settings[option]
        end
    end

    function BTP:GetHUDOption(hud, option)
        if hud and option then
            return self.settings.hud[hud][option]
        end
    end

    function BTP:GetColorFromTable(value)
        if value and value.r and value.g and value.b then
            return Color(255, value.r, value.g, value.b) / 255
        end
        return Color.white
    end

    function BTP:IsOr(string, ...)
        for i = 1, select("#", ...) do
            if string == select(i, ...) then
                return true
            end
        end
        return false
    end

    local ping_colors = {}
    function BTP:GetPingColor(ping, desync)
        if desync then
            return ping_colors[5]
        end
        local pos
        if ping <= 100 then
            pos = 0
        elseif ping >= 500 then
            pos = 4
        else
            pos = math.clamp(math.floor(ping / 100), 0, 4)
        end
        return ping_colors[pos]
    end

    function BTP:SetPingColor()
        if self:GetOption("ping_counter_color") then
            for i = 0, 4, 1 do
                ping_colors[i] = self:GetColorFromTable(self.settings["ping_counter_color_" .. i])
            end
        else
            for i = 0, 4, 1 do
                ping_colors[i] = Color.white
            end
        end
        ping_colors[5] = Color.red
    end

    function BTP:RoundNumber(n, bracket)
        bracket = bracket or 1
        local sign = n >= 0 and 1 or -1
        return math.floor(n / bracket + sign * 0.5) * bracket
    end

    function BTP:Log(s, log_type)
        log_type = log_type or self.Enum.LogType.Normal
        if log_type == self.Enum.LogType.Normal then
            log("[BTP] " .. (s or "nil"))
        else
            log("[BTP Warning] " .. (s or "nil"))
        end
    end

    BTP:Load()
end