local BTP = BTP
dofile(BTP.LuaPath .. "compatibility.lua")
dofile(BTP.LuaPath .. "common.lua")

BTP:Hook(HUDManager, "_create_teammates_panel", function(self, ...)
    BTP:SetPingColor()
    BTP:ApplyCompatibility()
    for i = 1, HUDManager.PLAYER_PANEL do
        self._teammate_panels[i]:InitBTP()
    end
    BTP:Log("Successfully initialized")
end)

BTP:Hook(HUDManager, "set_max_stamina", function(self, value)
    self._teammate_panels[HUDManager.PLAYER_PANEL]:SetMaxStamina(value)
end)

if BTP:GetOption("panel_style_position") == 3 and not (WolfHUD and WolfHUD:getSetting({"CustomHUD", "ENABLED"}, true) and HUDManager.CUSTOM_TEAMMATE_PANELS) then
    function HUDManager:teampanels_height()
        return 240
    end
end

if BTP:GetOption("show_stamina_counter") then
    local _f_set_stamina_value = HUDManager.set_stamina_value
    function HUDManager:set_stamina_value(value)
        _f_set_stamina_value(self, value)
        self._teammate_panels[HUDManager.PLAYER_PANEL]:SetStaminaValue(value)
    end
end

function HUDManager:GetTeammatePanelFromPeer(peer_id)
    for panel_id, panel in pairs(self._teammate_panels or {}) do
        if panel._peer_id == peer_id then
            return panel_id
        end
    end
end

local _f_add_teammate_panel = HUDManager.add_teammate_panel
function HUDManager:add_teammate_panel(character_name, player_name, ai, peer_id)
    local i = _f_add_teammate_panel(self, character_name, player_name, ai, peer_id)
    if i then
        self._teammate_panels[i]:SetPingVisibility(not ai)
    end
    return i
end

function HUDManager:SetSkillActive(i, skill, t, t_cooldown)
    local panel = self._teammate_panels[i]
    if panel and panel.ActivateArmorBreakInvulnerability and panel.SetArmorBreakInvulnerabilityVisibility then
        panel:ActivateArmorBreakInvulnerability(t)
        panel:SetArmorBreakInvulnerabilityVisibility(true, t_cooldown)
    end
end

if not Global.game_settings.single_player and BTP:GetOption("show_ping_counter") then -- Optimization, useless to compute in Single Player
    local update = HUDManager.update
    local update_rate = BTP:GetOption("ping_counter_update_rate")
    local last_ping_update = 0
    function HUDManager:update(t, dt)
        update(self, t, dt)
        if last_ping_update + update_rate < t then
            last_ping_update = t
            local peers = managers.network:session() and managers.network:session():peers()
            for _, peer in pairs(peers or {}) do
                if peer and peer:id() and peer:rpc() then
                    local panel = self:GetTeammatePanelFromPeer(peer:id())
                    if panel then
                        local qos = peer:qos()
                        local packet_loss = qos.packet_loss and qos.packet_loss > 0
                        self._teammate_panels[panel]:SetPingText(math.ceil(qos.ping), packet_loss)
                    end
                end
            end
        end
    end
end